@@ -22,23 +22,167 @@
  ***************************************************************************/
 
 #include <libvpd-2/helper_functions.hpp>
+#include <libvpd-2/logger.hpp>
 #include <cstdio>
 #include <fcntl.h>
 #include <unistd.h>
 #include <cstdlib>
 #include <cstring>
 #include <fstream>
+#include <errno.h>
+#include <string.h>
+#include <assert.h>
+#include <sstream>
+#include <sys/wait.h>
+#include <linux/limits.h>
+#include <libgen.h>
 
 
-#define BUF_SIZE 512
+#define BUF_SIZE PATH_MAX
 
 using namespace std;
 using namespace lsvpd;
 
+static int process_child(char *argv[], int pipefd[])
+{
+	int     nullfd;
+
+	close(pipefd[0]);
+
+	/* stderr to /dev/null redirection */
+	nullfd = open("/dev/null", O_WRONLY);
+	if (nullfd == -1) {
+		log_notice("Failed to open \'/dev/null\' for redirection "
+			   "(%d:%s).", errno, strerror(errno));
+		close(pipefd[1]);
+		return -1;
+	}
+
+	/* redirect stdout to write-end of the pipe */
+	if (dup2(pipefd[1], STDOUT_FILENO) == -1) {
+		log_notice("Failed to redirect pipe write fd to stdout "
+			   "(%d:%s).", errno, strerror(errno));
+		goto err_out;
+	}
+
+	if (dup2(nullfd, STDERR_FILENO) == -1) {
+		log_notice("Failed to redirect \'/dev/null\' to stderr "
+			   "(%d:%s).", errno, strerror(errno));
+		goto err_out;
+	}
+
+	execve(argv[0], argv, NULL);
+	/* some failure in exec */
+
+err_out:
+	close(pipefd[1]);
+	close(nullfd);
+	return -1;
+}
+
+/*
+ * This function mimics popen(3).
+ *
+ * Returns:
+ *   NULL, if fork(2), pipe(2) and dup2(3) calls fail
+ *
+ * Note:
+ *   fclose(3) function shouldn't be used to close the stream
+ *   returned here, since it doesn't wait for the child to exit.
+ */
+FILE *spopen(char *argv[], pid_t *ppid)
+{
+	FILE    *fp = NULL;
+	int     pipefd[2];
+	pid_t   cpid;
+
+	if (argv == NULL)
+		return fp;
+
+	if (access(argv[0], F_OK|X_OK) != 0) {
+		log_notice("The command \"%s\" is not executable.", argv[0]);
+		return fp;
+	}
+
+	if (pipe(pipefd) == -1) {
+		log_notice("Failed in pipe(), error: %d:%s",
+			   errno, strerror(errno));
+		return NULL;
+	}
+
+	cpid = fork();
+	switch (cpid) {
+	case -1:
+		/* Still in parent; Failure in fork() */
+		log_notice("fork() failed, error : %d:%s",
+			   errno, strerror(errno));
+		close(pipefd[0]);
+		close(pipefd[1]);
+		return NULL;
+	case  0: /* Code executed by child */
+		if (process_child(argv, pipefd) == -1) {
+			log_notice("Error occured while processing write end "
+				   "of the pipe (in child).");
+			exit(EXIT_FAILURE);
+		}
+		exit(EXIT_SUCCESS);
+	default: /* Code executed by parent */
+		/* store the child pid for pclose() */
+		*ppid = cpid;
+
+		close(pipefd[1]);
+
+		fp = fdopen(pipefd[0], "r");
+		if (fp == NULL) {
+			log_notice("fdopen() error : %d:%s",
+				   errno, strerror(errno));
+			close(pipefd[0]);
+			return NULL;
+		}
+		break;
+	}
+
+	return fp;
+}
+
+/*
+ * This function closely mimics pclose(3).
+ * Returns :
+ *   On success :  exit status of the command as returned by waitpid(2),
+ *   On failure : -1 if waitpid(2) returns an error.
+ *   If it cannot obtain the child status, errno is set to ECHILD.
+ */
+int spclose(FILE *stream, pid_t cpid)
+{
+	int     status;
+	pid_t   pid;
+
+	/*
+	 * Close the stream, fclose() takes care of closing
+	 * the underlying fd.
+	 */
+	if (fclose(stream) == EOF) {
+		log_notice("Failed in fclose() (%d:%s)",
+			   errno, strerror(errno));
+		return -1;
+	}
+
+	/* Wait for the child to exit */
+	do {
+		pid = waitpid(cpid, &status, 0);
+	} while (pid == -1 && errno == EINTR);
+
+	/* Couldn't obtain child status */
+	if (status == -1)
+		errno = SIGCHLD;
+
+	return status;
+}
+
 	/**
 	 * findAIXFSEntry
-	 * @brief Finds AIX name which exists as a file or link in the dir 
-	 * 	'rootPath'
+	 * @brief Finds AIX name which exists as a file or link in the dir
+	 *	'rootPath'
 	 */
 	string HelperFunctions::findAIXFSEntry(vector <DataItem*> aixNames,
 											const string& rootPath)
@@ -48,7 +192,7 @@ using namespace lsvpd;
 		vector<DataItem*>::const_iterator i, end;
 		i = aixNames.begin();
 		end = aixNames.end();
-		
+
 		while (i != end) {
 			fin = rootPath + (*i)->getValue();
 			fd = open( fin.c_str( ), O_RDONLY | O_NONBLOCK );
@@ -66,24 +210,45 @@ using namespace lsvpd;
  * parseString
  * @brief Parses out strings based on a count of position [0...x],
  *  with beginning and end of each string given by a '"'.
- * Ex: '"string 0" ... "string 1" ... "string 2" ...
+ * Ex: "string 0" ... "string 1" ... "string 2" ...
+ * If position is less/more than the string available, then it logs
+ * an error and returns empty string.
  */
 string HelperFunctions::parseString(const string& line, int str_pos, string& out)
 {
-	unsigned int pos = 0;
-	int beg, len, count = 0;
+        size_t pos = 0, beg = 0;
+	int str_pos_prov = str_pos;
 
-	do {
-		beg = line.find('"', pos) + 1;
-		pos = line.find('"', beg) + 1;
-		len = pos - beg;
-		count++;
-		} while  ((pos != string::npos) && (count < str_pos));
+	if (line == "")
+		return string();
 
-	out = line.substr(beg, len - 1);
-	return out;
+        if (str_pos <= 0) {
+		log_info("Invalid position : %d", str_pos);
+                return string();
+        }
+
+        while (str_pos > 0) {
+                beg = line.find('"', pos);
+                if (beg == string::npos) {
+			log_info("String not found at position: %d",
+				 (str_pos_prov - str_pos) + 1);
+                        return string();
+                }
+
+                pos = line.find('"', beg + 1);
+                if (pos == string::npos) {
+			log_info("String at position %d not terminated properly",
+				 (str_pos_prov - str_pos) + 1);
+                        return string();
+                }
+                pos++;
+                str_pos--;
+	}
+
+        out = line.substr(beg + 1, pos - beg - 2);
+        return out;
 }
-	
+
 	/**
 	 * parsePathr
 	 * @brief Reverse parse of a path, returning the node specified by 'count'
@@ -97,17 +262,17 @@ string HelperFunctions::parseString(const string& line, int str_pos, string& out
 	{
 		int beg, end;
 		int i = count;
-		
+
 		if( path == "" )
 			return path;
-		
+
 		beg = end = path.length();
-		
+
 		if (path[end] == '/')
 			beg--;
-		
+
 		cout << "String: " << path << endl;
-		
+
 		while (i >= 0) {
 			end = beg - 1;
 			cout << "RFind Bef: beg: " << beg << ", end: " << end << ", i: " << i << endl;
@@ -115,10 +280,10 @@ string HelperFunctions::parseString(const string& line, int str_pos, string& out
 			cout << "RFind : beg: " << beg << ", end: " << end << ", i: " << i << endl;
 			i--;
 		}
-		
+
 		return path.substr(beg, end - beg);
 	}
-	
+
 	/**
 	 * parsePath
 	 * @brief Parse of a path, returning the node specified by 'count'
@@ -130,39 +295,38 @@ string HelperFunctions::parseString(const string& line, int str_pos, string& out
 	 */
 	string HelperFunctions::parsePath(const string& path, int count)
 	{
-		int beg = 0, end, i = count;
-		
+		int beg = 0, end = 0, i = count;
+
 		if( path == "" )
 			return path;
-		
+
 		if (path[end] == '/')
 			end--;
-		
+
 		while (i >= 0) {
 			beg = path.find("/", beg) + 1;
 			i--;
 		}
-		
+
 		end = path.find("/", beg);
-		
+
 		return path.substr(beg, end - beg);
 	}
-	
+
 	/**
 	 * getSymLinkTarget
-	 * @brief Returns the absolute path pointed at by a sym link
+	 * @brief Returns the absolute path of regular file pointed at by a sym link
 	 * @param full path of sym link
 	 * @return String abs target
 	 */
 	string HelperFunctions::getSymLinkTarget(const string& symLinkPath)
 	{
-		char linkTarget[BUF_SIZE], *buf;
-		
-		buf = strdup(symLinkPath.c_str());
-		int len = readlink(buf, linkTarget, sizeof(
-			linkTarget));
-		linkTarget[len] = '\0';
-		return string(getAbsolutePath(linkTarget, buf));
+		char linkTarget[PATH_MAX];
+
+		if ( realpath( symLinkPath.c_str(), linkTarget ) == NULL )
+			return string();
+
+		return string( linkTarget );
 	}
 
 /* Drops one dir entry on the end of a path
@@ -199,12 +363,12 @@ string HelperFunctions::parseString(const string& line, int str_pos, string& out
 		int i, ret = 0;
 		char *filePath;
 
+		if (len <= 0)
+			return 0;
+
 		filePath = strdup(filePath_t.c_str());
-
-		if (len <= 0) {
-			free(filePath);
+		if (filePath == NULL)
 			return 0;
-		}
 
 		/* erase filenames */
 		i = len;
@@ -224,32 +388,39 @@ string HelperFunctions::parseString(const string& line, int str_pos, string& out
 		free(filePath);
 		return ret;
 	}
-	
-	/* @brief Drops dirs from curDir, 1 for each '../', then add remainder of filePath
-	 * to curDir */
+
+	/* getAbsolutePath
+	 * @brief It returns the absolute Path of Destination string(file/Dir)
+	 * where symlink points to.
+	 * @param relative path of destination directory w.r.t curDir
+	 * @param Absolute path of symlink(file/folder)
+	 * @return Absolute path of Destination Directory(i.e relPath), where
+	 * symlink(curDir) points to.
+	 */
 	string HelperFunctions::getAbsolutePath(char * relPath, char * curDir)
 	{
-		int len = strlen(relPath);
-		int i = strlen(curDir);
-		string tmp;
+		/* Converting it to string class object, to escape NULL checking */
+		string relPath_s(relPath), curDir_s(curDir);
 
-        if (len <= 0 || i <= 0)
-        	return string();
+		/* If string is empty */
+		if ( relPath_s.empty() || curDir_s.empty() )
+			return string();
 
-		while (i > 0 && curDir[i] != '/')
-			curDir[i--] = '\0';
+		/* If relPath_s contains absolute path, then return relPath_s */
+		if ( relPath_s[0] == '/' )
+			return relPath_s;
 
-		i = 0;
-		while (strncmp(&relPath[i * 3], "../", 3) == 0) {
-        	dropDir(curDir);
-        	i++;
-        }
+		curDir_s =  dirname((char *)curDir_s.c_str());
 
-        tmp = string(curDir) + string(&relPath[i * 3]);
+		string rpath = curDir_s + '/' + relPath_s;
+		char linkTarget[PATH_MAX];
 
-        return tmp;
+		if ( realpath( rpath.c_str(), linkTarget ) == NULL )
+			return string();
+
+		return string( linkTarget );
 	}
-	
+
 	/**
 	 * Takes two strings and finds if the two are equivalent.
 	 * s1 can contain '*', meaning zero or more of anything can be
@@ -257,7 +428,7 @@ string HelperFunctions::parseString(const string& line, int str_pos, string& out
 	 */
 	bool HelperFunctions::matches(const string& s1, const string& s2)
 	{
-		unsigned int beg = 0, end = s1.length();
+		unsigned int beg = 0, end;
 		unsigned int z;
 //		coutd << " s1 = " << s1 << ", s2 = " << s2 << endl;
 
@@ -307,16 +478,17 @@ string HelperFunctions::parseString(const string& line, int str_pos, string& out
 	void HelperFunctions::str2chr(char **str1, const string& str2)
 		{
 			unsigned int i;
-			*str1 = new char[ str2.length( ) * sizeof( char ) ];
+			*str1 = new char[ str2.length( ) * sizeof( char ) + 1 ];
+			if ( *str1 != NULL ) {
+				i=0;
+				while (i < str2.length())
+				{
+					(*str1)[i] = str2[i];
+					i++;
+				}
 
-			i=0;
-			while (i < str2.length())
-			{
-				(*str1)[i] = str2[i];
-				i++;
+				(*str1)[i] = '\0';
 			}
-
-			(*str1)[i] = '\0';
 		}
 
 	/* @brief Ensure a passed in fs path arg is in reasonable format */
@@ -330,52 +502,70 @@ string HelperFunctions::parseString(const string& line, int str_pos, string& out
 			str[i] = '\0';
 	}
 
-	/* @brief Ensure a passed in fs path arg is in reasonable format */
+	/* @brief Ensure a passed in fs path arg is in reasonable format
+	 * It mainly checks for '/' in the end.
+	 * @param e.g: /proc/device-tree
+	 */
 	char *HelperFunctions::fs_fixPath(char *path_t)
 	{
 		char *path;
-		char *chr;
+		char *chr = NULL;
+
+		if ( path_t == NULL )
+			return NULL;
 
 		path = strdup(path_t);
-		/* Basic wierdness checking*/
-			chr = &path[strlen(path)];
-			while (chr != '\0')
-				chr--;
-			if (*chr == '/')
-				*chr = '\0';
+		if ( path == NULL )
+			return NULL;
+
+		/* Basic weirdness checking */
+		chr = &path[strlen(path) - 1];
+		if ( *chr == '/' )
+			*chr = '\0';
 
 		return path;
 	}
-	
+
 	bool HelperFunctions::file_exists(const string& file)
 	{
-		FILE *f = fopen(file.c_str(), "r");
-		if (!f) return false;
-		fclose(f);
-		return true;
+		/* If file is available and have read permission */
+		if (access(file.c_str(), R_OK) == 0)
+			return true;
+
+		/* If file is not present - caller will create it, so no need
+		 * to log error. But if file is present but read access call
+		 * gets failed due to other errors condition except(ENOENT),
+		 * then error should be logged.
+		 */
+		if (errno != ENOENT)
+			log_err("Failed to access file %s : %s", file.c_str(),
+				strerror(errno));
+
+		return false;
 	}
 
 /** readMatchFromFile
- *  @brief Returns first matching line from specified file, 
+ *  @brief Returns first matching line from specified file,
  *    in which was found a match to the specified string
-*/ 
-string HelperFunctions::readMatchFromFile(const string& file,
-											const string& str)
+*/
+string HelperFunctions::readMatchFromFile(const string& file, const string& str)
 {
 	char buf[BUF_SIZE];
 	string line;
-	
+
 	if (file_exists(file)) {
 		ifstream fin(file.c_str());
-		
+
 		while(!fin.eof()) {
 			fin.getline(buf,BUF_SIZE - 1);
 			line = string(buf);
 			int loc = line.find(str);
 			if (loc == 0) {
+				fin.close();
 				return line;
 			}
 		}
+		fin.close();
 		return string("");
 	}
 	return string("");
@@ -412,14 +602,39 @@ string HelperFunctions::readMatchFromFile(const string& file,
 	int HelperFunctions::execCmd( const char *cmd, string& output )
 	{
 		char buf[BUF_SIZE];
-		FILE *fp = popen(cmd, "r");
+		char *system_args[32] = {NULL,};
+		pid_t cpid;
+		int i = 0;
+		int rc = -1;
+		istringstream ss(cmd);
+		string s;
+		FILE *fp;
 
+		while ( ss >> s ) {
+			if (i < 32) {
+				system_args[i] = strdup(s.c_str());
+				if (system_args[i] == NULL)
+					goto free_mem;
+				i++;
+			} else {
+				goto free_mem;
+			}
+		}
+
+		fp = spopen(system_args, &cpid);
 		if (fp == NULL)
-			return 1;
+			goto free_mem;
 
 		while ( !feof(fp) )
 			if (fgets(buf, BUF_SIZE - 1, fp) != NULL)
 				output += buf;
-		pclose(fp);
-		return 0;
+
+		spclose(fp, cpid);
+		rc = 0;
+
+free_mem:
+		for (i -= 1; i >= 0; i--)
+			free(system_args[i]);
+
+		return rc;
 	}
