@@ -29,9 +29,14 @@
 #include <string>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <sys/wait.h>
 #include <time.h>
 #include <cstdlib>
 #include <errno.h>
+#include <fcntl.h>
+#include <stdio.h>
+#include <string.h>
+#include <unistd.h>
 
 using namespace std;
 
@@ -44,13 +49,12 @@ namespace lsvpd
 	VpdRetriever::VpdRetriever( string envDir,
 		string dbFileName ) throw( VpdException& )
 	{
-		db = new VpdDbEnv( envDir, dbFileName, true );
-
-		if( db == NULL )
-		{
-			Logger logger;
-			logger.log( "Out of memory, failed to build VpdEnv.", LOG_ERR );
-			VpdException ve( "Out of memory, failed to build VpdEnv." );
+		try {
+			db = new VpdDbEnv( envDir, dbFileName, true );
+		}
+		catch (std::bad_alloc& ba) {
+			log_err("Out of memory, failed to build VpdEnv.");
+			VpdException ve("Out of memory, failed to build VpdEnv.");
 			throw ve;
 		}
 	}
@@ -87,18 +91,66 @@ namespace lsvpd
 				 * after the VPD db was created/modified. So
 				 * run vpdupdate.
 				 */
+				pid_t cpid;	/* Pid of child */
+				int rc;		/* Holds return value */
+				int status;	/* exit value of child */
+
 				logger.log( "libvpd: Running vpdupdate to update the default db.", LOG_INFO );
-				system( "vpdupdate >/dev/null 2>&1" );
+				cpid = fork();
+				if (cpid == -1) {	/* fork failed */
+					logger.log("libvpd: Fork failed, while running vpdupdate\n", LOG_INFO);
+					VpdException ve( "libvpd: Fork failed, while running vpdupdate\n" );
+					throw ve;
+				} else if (cpid == 0) {	/* child */
+					char *system_arg[2];
+					int nullfd;
+
+					system_arg[0] = (char *)"/usr/sbin/vpdupdate";
+					system_arg[1] = NULL;
+
+					nullfd = open("/dev/null", O_WRONLY);
+					if (nullfd == -1) {
+						log_notice("Failed to open /dev/null "
+							   "(%d:%s)", errno,
+							   strerror(errno));
+						exit(-2);
+					}
+
+					if (dup2(nullfd, STDOUT_FILENO) == -1) {
+						log_notice("Failed to redirect stderr "
+							   "to /dev/null (%d:%s)",
+							   errno, strerror(errno));
+						 close(nullfd);
+						 exit(-2);
+					}
+
+					if (dup2(nullfd, STDERR_FILENO) == -1) {
+						log_notice("Failed to redirect stdout "
+							   "to /dev/null (%d:%s)",
+							   errno, strerror(errno));
+						 close(nullfd);
+						 exit(-2);
+					}
+
+					execv(system_arg[0], system_arg);
+					exit(-2);
+				} else {	/* parent */
+					rc = waitpid(cpid, &status, 0);
+					if (rc == -1)
+						logger.log("libvpd: wait failed, while running vpdupdate\n", LOG_INFO);
+					if (WIFEXITED(status) &&
+					    (signed char)WEXITSTATUS(status) == -2)
+						logger.log("libvpd: execv failed\n", LOG_INFO);
+				}
 			}
 		}
-		db = new VpdDbEnv( VpdRetriever::DEFAULT_DIR,
-					VpdRetriever::DEFAULT_FILE, true );
-
-		if( db == NULL )
-		{
-			Logger logger;
-			logger.log( "Out of memory, failed to build VpdEnv.", LOG_ERR );
-			VpdException ve( "Out of memory, failed to build VpdEnv." );
+		try {
+			db = new VpdDbEnv( VpdRetriever::DEFAULT_DIR,
+					   VpdRetriever::DEFAULT_FILE, true );
+		}
+		catch (std::bad_alloc& ba) {
+			log_err("Out of memory, failed to build VpdEnv.");
+			VpdException ve("Out of memory, failed to build VpdEnv.");
 			throw ve;
 		}
 	}
